from django.contrib import admin
from djangovoice.models import Feedback, Status, Type


class SlugFieldAdmin(admin.ModelAdmin):
    prepopulated_fields = {"slug": ("title",)}


admin.site.register(Feedback)
admin.site.register([Status, Type], SlugFieldAdmin)



from django.template import Library

register = Library()

@register.simple_tag
def user_name(user):
    """if user has full name, get user's full name, else username.
    """
    full_name = user.get_full_name()
    if not full_name:
        return user.username

    return full_name

@register.inclusion_tag('djangovoice/tags/widget.html', takes_context=True)
def djangovoice_widget(context):
    arguments = {'STATIC_URL': context.get('STATIC_URL')}

    return arguments



from django.conf.urls.defaults import *
from django.contrib import admin
admin.autodiscover()
from voting.views import vote_on_object
from djangovoice.models import Feedback
from djangovoice.views import *
from djangovoice.feeds import LatestFeedback


feedback_dict = {
    'model': Feedback,
    'template_object_name': 'feedback'
}

# NOTE: can we do something for pep8 here? lines are too long.
urlpatterns = patterns(
    '',
    url(r'^$', view=FeedbackListView.as_view(), name='djangovoice_home'),
    url(r'^(?P<list>all|open|closed|mine)/$', view=FeedbackListView.as_view(), name='djangovoice_list'),
    url(r'^(?P<list>all|open|closed|mine)/(?P<type>[-\w]+)/$', view=FeedbackListView.as_view(), name='djangovoice_list_type'),
    url(r'^(?P<list>all|open|closed|mine)/(?P<type>[-\w]+)/(?P<status>[-\w]+)/$', view=FeedbackListView.as_view(), name='djangovoice_list_type_status'),
    url(r'^widget/$', view=FeedbackWidgetView.as_view(), name='djangovoice_widget'),
    url(r'^submit/$', view=FeedbackSubmitView.as_view(), name='djangovoice_submit'),
    url(r'^(?P<pk>\d+)/$', view=FeedbackDetailView.as_view(), name='djangovoice_item'),
    url(r'^(?P<pk>\d+)/edit/$', view=FeedbackEditView.as_view(), name='djangovoice_edit'),
    url(r'^(?P<pk>\d+)/delete/$', view=FeedbackDeleteView.as_view(), name='djangovoice_delete'),
    url(r'^(?P<object_id>\d+)/(?P<direction>up|down|clear)/?$', vote_on_object, feedback_dict, name='djangovoice_vote'),
    url(r'^feeds/latest/$', view=LatestFeedback(), name='feeds_latest'),
)



from django import forms
from django.forms import ModelForm
from djangovoice.models import Feedback


class WidgetForm(ModelForm):
    class Meta:
        model = Feedback
        fields = ('type', 'anonymous', 'private', 'title', 'description',)


class EditForm(ModelForm):
    class Meta:
        model = Feedback



from django.db import models
from django.contrib import admin
from django.contrib.auth.models import User

STATUS_CHOICES = (
    ('open', 'Open'),
    ('closed', 'Closed'),
)


class Status(models.Model):
    title = models.CharField(max_length=500)
    slug = models.SlugField(max_length=500)
    default = models.BooleanField(
        blank=True,
        help_text='New feedback will have this status')
    status = models.CharField(
        max_length=10, choices=STATUS_CHOICES, default="open")

    def save(self):
        if self.default == True:
            try:
                default_project = Status.objects.get(default=True)
                default_project.default = False
                default_project.save()
            except:
                pass
        super(Status, self).save()

    def __unicode__(self):
        return self.title

    class Meta:
        verbose_name_plural = "statuses"


class Type(models.Model):
    title = models.CharField(max_length=500)
    slug = models.SlugField(max_length=500)

    def __unicode__(self):
        return self.title


class Feedback(models.Model):
    type = models.ForeignKey(Type)
    title = models.CharField(max_length=500)
    description = models.TextField(
        blank=True,
        help_text='This wiill be viewable by other people - '\
                  'do not include any private details such as '\
                  'passwords or phone numbers here.')
    anonymous = models.BooleanField(
        blank=True,
        help_text='Do not show who sent this')
    private = models.BooleanField(
        blank=True,
        help_text='Hide from public pages. Only site administrators '\
                  'will be able to view and respond to this.')
    user = models.ForeignKey(User, blank=True, null=True)
    created = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    status = models.ForeignKey(Status)
    duplicate = models.ForeignKey('self', null=True, blank=True)

    def save(self, *args, **kwargs):
        try:
            self.status
        except:
            try:
                default = Status.objects.get(default=True)
            except:
                default = Status.objects.filter()[0]
            self.status = default
        super(Feedback, self).save()

    def get_absolute_url(self):
        return ('djangovoice_item', (self.id,))
    get_absolute_url = models.permalink(get_absolute_url)

    def __unicode__(self):
        return self.title

    class Meta:
        verbose_name_plural = "feedback"



from django.utils import unittest
from djangovoice.models import *


class StatusTestCase(models.Model):
    def setUp(self):
        self.in_progress = Status.objects.create(
            title='In progress', slug='in_progress', default=False)
        self.need_to_test = Status.objects.create(
            title='Need to test', slug='need_to_test', default=True)

    def testSpeaking(self):
        self.assertEqual(self.in_progress.status, 'open')
        self.assertEqual(self.need_to_test.default, True)


class TypeTestCase(unittest.TestCase):
    def setUp(self):
        self.bug = Type.objects.create(title='Bug', slug='bug')
        self.betterment = Type.objects.create(title='Betterment',
                                              slug='betterment')

    def testSpeaking(self):
        self.assertEqual(self.bug.slug, 'bug')
        self.assertEqual(self.betterment.title, 'Betterment')


class FeedbackTestCase(unittest.TestCase):
    def setUp(self):
        feedback_type = Type.objects.create(title='Bug', slug='bug')
        feedback_user = User.objects.create_user(
            username='djangovoice', email='django@voice.com')
        self.login_form_does_not_work = Feedback.objects.create(
            type=feedback_type,
            title='Login form does not work.',
            description='What a fucking test...',
            anonymous=False,
            private=True,
            user=feedback_user)

    def testSpeaking(self):
        default_status = Status.objects.get(default=True)
        self.assertEqual(self.login_form_does_not_work.status, default_status)



from django.contrib.syndication.views import Feed
from djangovoice.models import Feedback


class LatestFeedback(Feed):
    title = "Feedback"
    link = "/feedback/"
    description = "Latest feedback"

    def items(self):
        return Feedback.objects.filter(private=False).order_by('-created')[:10]



from django.contrib import messages
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.http import Http404
from django.utils.translation import ugettext as _
from djangovoice.models import Feedback, Type
from djangovoice.forms import *
from djangovoice.utils import paginate

# generic views
from django.views.generic.base import TemplateView
from django.views.generic.edit import DeleteView
from django.views.generic.edit import FormView
from django.views.generic.detail import DetailView

# decorators
from django.contrib.auth.decorators import login_required
from django.utils.decorators import method_decorator


class FeedbackDetailView(DetailView):

    template_name = 'djangovoice/detail.html'
    model = Feedback

    def get(self, request, *args, **kwargs):
        feedback = self.get_object()

        if feedback.private:
            if not request.user.is_staff and request.user != feedback.user:
                return Http404

        return super(FeedbackDetailView, self).get(request, *args, **kwargs)


# FIXME: Can not we use ListView?
class FeedbackListView(TemplateView):

    template_name = 'djangovoice/list.html'

    def get_context_data(self, **kwargs):
        context = super(FeedbackListView, self).get_context_data(**kwargs)
        feedback = Feedback.objects.all().order_by('-created')
        feedback_list = kwargs.get('list', 'open')
        feedback_type = kwargs.get('type', 'all')
        feedback_status = kwargs.get('status', 'all')

        if feedback_list == 'open':
            title = _("Open Feedback")
            feedback = feedback.filter(status__status='open')
        elif feedback_list == 'closed':
            title = _("Closed Feedback")
            feedback = feedback.filter(status__status='closed')
        elif feedback_list == 'mine':
            title = _("My Feedback")
            feedback = feedback.filter(user=self.request.user)
        else:
            title = _("Feedback")

        if feedback_type != 'all':
            feedback = feedback.filter(type__slug=feedback_type)

        if feedback_status != 'all':
            feedback = feedback.filter(status__slug=feedback_status)

        if not self.request.user.is_staff:
            feedback = feedback.filter(private=False)

        feedback_page = paginate(feedback, 10, self.request)

        context.update({
                'feedback_list': feedback_page.object_list,
                'pagination': feedback_page,
                'list': feedback_list,
                'status': feedback_status,
                'type': feedback_type,
                'navigation_active': feedback_list,
                'title': title})

        return context

    def get(self, request, *args, **kwargs):
        if kwargs.get('list') == 'mine' and not request.user.is_authenticated():
            return HttpResponseRedirect(
                '%s?next=%s' % (reverse('django.contrib.auth.views.login'),
                                request.path))
        return super(FeedbackListView, self).get(request, *args, **kwargs)


class FeedbackWidgetView(FormView):

    template_name = 'djangovoice/widget.html'
    form_class = WidgetForm
    initial = {'type': Type.objects.get(pk=1)}

    def get(self, request, *args, **kwargs):
        return super(FeedbackWidgetView, self).get(request, *args, **kwargs)

    @method_decorator(login_required)
    def post(self, request, *args, **kwargs):
        return super(FeedbackWidgetView, self).post(request, *args, **kwargs)

    def form_valid(self, form):
        feedback = form.save(commit=False)
        if form.cleaned_data.get('anonymous') != 'on':
            feedback.user = self.request.user
        feedback.save()

        success_url = reverse('djangovoice_item', args=[feedback.pk])
        messages.add_message(
            self.request, messages.SUCCESS, _("Thanks for feedback."))

        return HttpResponseRedirect(reverse('djangovoice_widget'))

    def form_invalid(self, form):
        messages.add_message(self.request, messages.ERROR,
                             _("Form is invalid."))

        return super(FeedbackWidgetView, self).form_invalid(form)


class FeedbackSubmitView(FormView):

    template_name = 'djangovoice/submit.html'
    form_class = WidgetForm

    @method_decorator(login_required)
    def get(self, request, *args, **kwargs):
        return super(FeedbackSubmitView, self).get(request, *args, **kwargs)

    @method_decorator(login_required)
    def post(self, request, *args, **kwargs):
        return super(FeedbackSubmitView, self).post(request, *args, **kwargs)

    def form_valid(self, form):
        feedback = form.save(commit=False)
        if form.data.get('anonymous') != 'on':
            feedback.user = self.request.user

        feedback.save()

        return HttpResponseRedirect(feedback.get_absolute_url())


class FeedbackEditView(FormView):

    template_name = 'djangovoice/edit.html'

    def get_form_class(self):
        feedback = self.get_object()
        if self.request.user.is_staff:
            return EditForm
        elif self.request.user == feedback.user:
            return WidgetForm
        return None

    def get_object(self):
        return Feedback.objects.get(pk=self.kwargs.get('pk'))

    def get_form_kwargs(self):
        kwargs = super(FeedbackEditView, self).get_form_kwargs()
        kwargs.update({'instance': self.get_object()})

        return kwargs

    @method_decorator(login_required)
    def get(self, request, *args, **kwargs):
        form_class = self.get_form_class()
        if not form_class:
            raise Http404

        return super(FeedbackEditView, self).get(request, *args, **kwargs)

    @method_decorator(login_required)
    def post(self, request, *args, **kwargs):
        return super(FeedbackEditView, self).post(request, *args, **kwargs)

    def form_valid(self, form):
        feedback = form.save()

        return HttpResponseRedirect(feedback.get_absolute_url())


class FeedbackDeleteView(DeleteView):

    template_name = 'djangovoice/delete.html'

    def get_object(self):
        return Feedback.objects.get(pk=self.kwargs.get('pk'))

    @method_decorator(login_required)
    def get(self, request, *args, **kwargs):
        # FIXME: should feedback user have delete permissions?
        feedback = self.get_object()
        if not request.user.is_staff and request.user != feedback.user:
            raise Http404

        return super(FeedbackDeleteView, self).get(request, *args, **kwargs)

    @method_decorator(login_required)
    def post(self, request, *args, **kwargs):
        feedback = self.get_object()
        feedback.delete()

        return HttpResponseRedirect(reverse('djangovoice_home'))



from django.core.paginator import Paginator, InvalidPage, EmptyPage


def paginate(queryset, items, request):
    paginator = Paginator(queryset, items)
    try:
        page = int(request.GET.get('page', '1'))
    except ValueError:
        page = 1
    try:
        queryset_list = paginator.page(page)
    except (EmptyPage, InvalidPage):
        queryset_list = paginator.page(paginator.num_pages)

    return queryset_list



from djangovoice.models import Type
from django.template import Library,Node

register = Library()

def build_type_list(parser, token):
    """
    {% get_type_list %}
    """
    return TypeObject()

class TypeObject(Node):
    def render(self, context):
        context['type_list'] = Type.objects.all()
        return ''

register.tag('get_type_list', build_type_list)



from djangovoice.models import Status
from django.template import Library, Variable, TemplateSyntaxError, Node

register = Library()

class StatusObject(Node):
    def __init__(self, list):
        self.list = Variable(list)
    
    def render(self, context):
        list = self.list.resolve(context)
        status_list = Status.objects.all()
        
        if list == "open":
            status_list = status_list.filter(status="open")
        elif list == "closed":
            status_list = status_list.filter(status="closed")
        
        context['status_list'] = status_list
        return ''

def build_status_list(parser, token):
    """
    {% get_status_list %}
    """
    bits = token.contents.split()
    if len(bits) != 2:
        raise TemplateSyntaxError, "'%s' tag takes exactly 1 arguments" % bits[0]
    return StatusObject(bits[1])

register.tag('get_status_list', build_status_list)



# -*- coding: utf-8 -*-

from django.http import HttpResponse
from django.utils import simplejson as json

def apply_only_xhr(original_function):
    def decorated(request, *args, **kwargs):
        if not request.is_ajax():
            return HttpResponse(status=403)

        return original_function(request, *args, **kwargs)

    return decorated

def return_json(original_function):
    def decorated(request, *args, **kwargs):
        response = original_function(request, *args, **kwargs)
        if not response:
            response = {'errors': True}
        else:
            response.update({'errors': False})

        data = json.dumps(response, indent=None).encode('utf-8')

        return HttpResponse(data, 'application/json')

    return decorated



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

djangovoice = __import__('djangovoice')
description = file('README.rst', 'r')

setup(
    name='django-voice',
    version=djangovoice.get_version(),
    description="A feedback application for Django 1.3 or later",
    long_description=description.read(),
    author=u'Gökmen Görgen',
    author_email='gokmen@alageek.com',
    url='https://github.com/gkmngrgn/django-voice',
    license='BSD',
    platforms='any',
    packages=find_packages(exclude=('demo', 'demo.*')),
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: Django",
        "Environment :: Web Environment"
    ],
    install_requires=[
        "Django>=1.3",
        "django-gravatar>=0.1.0",
        "django-voting>=0.1"
    ]
)



__version_info__ = {
    'major': 0,
    'minor': 3,
    'micro': 3,
    'releaselevel': 'final'
}


def get_version():
    """
    Return the formatted version information
    """
    vers = ["%(major)i.%(minor)i" % __version_info__, ]

    if __version_info__['micro']:
        vers.append(".%(micro)i" % __version_info__)
    if __version_info__['releaselevel'] != 'final':
        vers.append('%(releaselevel)s' % __version_info__)
    return ''.join(vers)

__version__ = get_version()
